import numpy as np
import matplotlib.pyplot as plt
from skimage.metrics import structural_similarity
from skimage.exposure import match_histograms


def img_gray(img):
    """
    Convert an image to grayscale.

    Args:
        img (ndarray): The input image array.

    Returns:
        ndarray: The grayscale image array.
    """
    # Convert the image array to grayscale
    gray_array = np.dot(img[..., :3], [0.2989, 0.5870, 0.1140])
    # Convert the array to uint8 data type
    gray_array = gray_array.astype(np.uint8)
    
    return gray_array


def Encontrando_diferença(img1,img2):
    """
    Calculates the difference between two images and returns the 
    normalized difference between two images.

    Parameters:
        img1 (numpy.ndarray): The first image.
        img2 (numpy.ndarray): The second image.

    Returns:
        numpy.ndarray: The normalized difference between the two images.
    """
    # primeiro comparar se os tamanhos sao iguais
    assert img1.shape == img2.shape , "as imagens devem ter o mesmo tamanho"
    # convertenso para a escala de siza
    img1 = img_gray(img1)
    img2 = img_gray(img2)
    
    score, diff = structural_similarity(img1,img2 ,full=True)
    print("A similaridade entre as imagens e: ", score)
    
    diff_normalizado  = (diff - np.min(diff)) / ( np.max(diff) - np.min(diff) )
    return diff_normalizado

def Mix_imagens(img1,img2,number=-1):
    """
    Generate a function comment for the given function body.

    :param img1: The first image to be mixed.
    :param img2: The second image to be mixed.
    :param number: The channel axis used for matching histograms, defaults to -1.

    :return: The image resulting from mixing img1 and img2 with matched histograms.
    """
    match_img = match_histograms(img1,img2,channel_axis=number)
    return match_img


import matplotlib.pyplot as plt

def plot_img(imagem, title="", tamanho=(12,4), mostra_eixos='off' ):
    """
        Plot de uma unica imagem com um tiulo e tamanho definido

        Args:
            imagem: The image to be plotted.
            title: Optional title for the plot (default is an empty string).
            tamanho: Optional tuple specifying the size of the plot (default is (12,4)).
            mostra_eixos: Optional parameter specifying whether to show the axes of the plot (default is 'off').

        Returns:
            imagem
    """
    plt.figure(figsize=tamanho)
    plt.title(title)
    plt.imshow(imagem)
    plt.axis(mostra_eixos)
    plt.show()
    
    
def plot_resultados(*args):
    """
        Plot de varias imagem com um tiulo e tamanho definido.
        
        Parameters:
            *args: Variable number of images to be plotted.

        Returns:
            None
    """
    numero_img = len(args)
    fig, eixo = plt.subplots(nrows=1, ncols=numero_img,figsize=(12,4))
    nome_lista = ['Imagem {}'.format(i+1) for i in range(numero_img)]
    nome_lista.append('Resultado')
    
    for ax,nome,imagem in zip(eixo,nome_lista,args):
        ax.set_title(nome)
        ax.imshow(imagem,cmap='gray')
        ax.axis('off') 
    
    fig.tight_layout()
    plt.show()   
    
def plot_hist(imagem):
    fig,axis = plt.subplots(nrows=1 , ncols=3, figsize=(12,4), sharex=True, sharey=True)
    
    col_lista = ['gray', 'red', 'green']
    for index, (ax, col) in enumerate(zip(axis, col_lista)): 
        ax.set_title('{} histograma'.format(col.title()))
        ax.hist(imagem[:,:,index].ravel(), bins=256, color=col, alpha=0.8)
    fig.tight_layout()
    plt.show()



from skimage import transform

def redimensionar_imagem(imagem, proporção):
    """
    Resizes an image according to a specified proportion.

    Parameters:
        imagem (numpy.ndarray): The input image to be resized.
        proporção (float): The proportion by which to resize the image. Must be between 0 and 1.

    Returns:
        numpy.ndarray: The resized image.

    Raises:
        AssertionError: If the proportion is not between 0 and 1.
    """
    assert  0 <= proporção <=1, " Proporção invalida, tente um valor entre 0 e 1"
    altura  =  imagem.shape[0]*proporção
    largura =  imagem.shape[1]*proporção
    nova_imagem = transform.resize(imagem, (altura,largura))
    return nova_imagem  


from setuptools import setup, find_packages

with open("README.md", "r") as f:
    long_description = f.read()
with open("requirements.txt", "r") as f:
    requerimentos = f.read().splitlines()


setup(
    name    = "processamento_de_img",
    version = "0.1",
    author  = "Nick: Oibug/Nome:Gubio Gomes De Lima",
    author_email    = 'gubiojogo@gmail.com',
    description     = " pacote simples para Processamento de imagem em Python",
    long_description    = long_description,
    long_description_content_type   = "text/markdown",
    packages    = find_packages(),
    url =    "https://github.com/GubioGL/BootCampDIO",
    install_requires = requerimentos,
    python_requires = '>=3.10',  
    )


from skimage.io import imread, imsave

def Ler_img(caminho,e_cinza=False):
    """
        Ler uma imagem em um caminho especificado.

        Parameters:
            caminho (str): The path to the image file.
            e_cinza (bool, optional): Whether to read the image in grayscale. Defaults to False.

        Returns:
            ndarray: The image read from the file.
    """
    imagem = imread(caminho,as_grey=e_cinza)
    return imagem

def Gravar_img(caminho,imagem):
    """
        Salvar a imagem em um caminho especificado.

        Parameters:
            imagem (ndarray): The image to be saved.
            caminho (str): The path to save the image to.

        Returns:
            None
    """
    imsave(caminho,imagem)